@@ -40,21 +40,6 @@ from taskflow.utils import misc
 
 LOG = logging.getLogger(__name__)
 
-UNCLAIMED_JOB_STATES = (
-    states.UNCLAIMED,
-)
-ALL_JOB_STATES = (
-    states.UNCLAIMED,
-    states.COMPLETE,
-    states.CLAIMED,
-)
-
-# Transaction support was added in 3.4.0
-MIN_ZK_VERSION = (3, 4, 0)
-LOCK_POSTFIX = ".lock"
-TRASH_FOLDER = ".trash"
-JOB_PREFIX = 'job'
-
 
 def check_who(meth):
     """Decorator that checks the expected owner type & value restrictions."""
@@ -88,13 +73,12 @@ class ZookeeperJob(base.Job):
             raise ValueError("Only one of 'book_data' or 'book'"
                              " can be provided")
         self._path = k_paths.normpath(path)
-        self._lock_path = self._path + LOCK_POSTFIX
+        self._lock_path = self._path + board.LOCK_POSTFIX
         self._created_on = created_on
         self._node_not_found = False
         basename = k_paths.basename(self._path)
         self._root = self._path[0:-len(basename)]
-        self._sequence = int(basename[len(JOB_PREFIX):])
-        self._last_state = None
+        self._sequence = int(basename[len(board.JOB_PREFIX):])
 
     @property
     def lock_path(self):
@@ -189,11 +173,7 @@ class ZookeeperJob(base.Job):
 
     @property
     def state(self):
-        current_state = self._fetch_state()
-        if self._last_state is not None:
-            states.check_job_transition(self._last_state, current_state)
-        self._last_state = current_state
-        return current_state
+        return self._fetch_state()
 
     def _fetch_state(self):
         owner = self.board.find_owner(self)
@@ -274,6 +254,16 @@ class ZookeeperJobBoardIterator(six.Iterator):
       over unclaimed jobs.
     """
 
+    _UNCLAIMED_JOB_STATES = (
+        states.UNCLAIMED,
+    )
+
+    _JOB_STATES = (
+        states.UNCLAIMED,
+        states.COMPLETE,
+        states.CLAIMED,
+    )
+
     def __init__(self, board, only_unclaimed=False, ensure_fresh=False):
         self._board = board
         self._jobs = collections.deque()
@@ -290,9 +280,9 @@ class ZookeeperJobBoardIterator(six.Iterator):
 
     def _next_job(self):
         if self.only_unclaimed:
-            allowed_states = UNCLAIMED_JOB_STATES
+            allowed_states = self._UNCLAIMED_JOB_STATES
         else:
-            allowed_states = ALL_JOB_STATES
+            allowed_states = self._JOB_STATES
         job = None
         while self._jobs and job is None:
             maybe_job = self._jobs.popleft()
@@ -325,24 +315,40 @@ class ZookeeperJobBoard(base.NotifyingJobBoard):
     Powered by the `kazoo <http://kazoo.readthedocs.org/>`_ library.
 
     This jobboard creates *sequenced* persistent znodes in a directory in
-    zookeeper (that directory defaults ``/taskflow/jobs``) and uses zookeeper
-    watches to notify other jobboards that the job which was posted using the
-    :meth:`.post` method (this creates a znode with contents/details in json)
-    The users of those jobboard(s) (potentially on disjoint sets of machines)
-    can then iterate over the available jobs and decide if they want to attempt
-    to claim one of the jobs they have iterated over. If so they will then
-    attempt to contact zookeeper and will attempt to create a ephemeral znode
-    using the name of the persistent znode + ".lock" as a postfix. If the
-    entity trying to use the jobboard to :meth:`.claim` the job is able to
-    create a ephemeral znode with that name then it will be allowed (and
-    expected) to perform whatever *work* the contents of that job that it
-    locked described. Once finished the ephemeral znode and persistent znode
-    may be deleted (if successfully completed) in a single transcation or if
-    not successfull (or the entity that claimed the znode dies) the ephemeral
+    zookeeper (that directory defaults to ``/taskflow/jobs``) and uses
+    zookeeper watches to notify other jobboards that the job which was posted
+    using the :meth:`.post` method (this creates a znode with contents/details
+    in json). The users of those jobboard(s) (potentially on disjoint sets of
+    machines) can then iterate over the available jobs and decide if they want
+    to attempt to claim one of the jobs they have iterated over. If so they
+    will then attempt to contact zookeeper and they will attempt to create a
+    ephemeral znode using the name of the persistent znode + ".lock" as a
+    postfix. If the entity trying to use the jobboard to :meth:`.claim` the
+    job is able to create a ephemeral znode with that name then it will be
+    allowed (and expected) to perform whatever *work* the contents of that
+    job described. Once finished the ephemeral znode and persistent znode may
+    be deleted (if successfully completed) in a single transaction or if not
+    successful (or the entity that claimed the znode dies) the ephemeral
     znode will be released (either manually by using :meth:`.abandon` or
-    automatically by zookeeper the ephemeral is deemed to be lost).
+    automatically by zookeeper when the ephemeral node and associated session
+    is deemed to have been lost).
     """
 
+    #: Transaction support was added in 3.4.0 so we need at least that version.
+    MIN_ZK_VERSION = (3, 4, 0)
+
+    #: Znode **postfix** that lock entries have.
+    LOCK_POSTFIX = ".lock"
+
+    #: Znode child path created under root path that contains trashed jobs.
+    TRASH_FOLDER = ".trash"
+
+    #: Znode **prefix** that job entries have.
+    JOB_PREFIX = 'job'
+
+    #: Default znode path used for jobs (data, locks...).
+    DEFAULT_PATH = "/taskflow/jobs"
+
     def __init__(self, name, conf,
                  client=None, persistence=None, emit_notifications=True):
         super(ZookeeperJobBoard, self).__init__(name, conf)
@@ -352,14 +358,14 @@ class ZookeeperJobBoard(base.NotifyingJobBoard):
         else:
             self._client = kazoo_utils.make_client(self._conf)
             self._owned = True
-        path = str(conf.get("path", "/taskflow/jobs"))
+        path = str(conf.get("path", self.DEFAULT_PATH))
         if not path:
             raise ValueError("Empty zookeeper path is disallowed")
         if not k_paths.isabs(path):
             raise ValueError("Zookeeper path must be absolute")
         self._path = path
         self._trash_path = self._path.replace(k_paths.basename(self._path),
-                                              TRASH_FOLDER)
+                                              self.TRASH_FOLDER)
         # The backend to load the full logbooks from, since whats sent over
         # the zookeeper data connection is only the logbook uuid and name, and
         # not currently the full logbook (later when a zookeeper backend
@@ -375,18 +381,22 @@ class ZookeeperJobBoard(base.NotifyingJobBoard):
         self._job_watcher = None
         # Since we use sequenced ids this will be the path that the sequences
         # are prefixed with, for example, job0000000001, job0000000002, ...
-        self._job_base = k_paths.join(path, JOB_PREFIX)
+        self._job_base = k_paths.join(path, self.JOB_PREFIX)
         self._worker = None
         self._emit_notifications = bool(emit_notifications)
         self._connected = False
 
     def _emit(self, state, details):
-        # Submit the work to the executor to avoid blocking the kazoo queue.
+        # Submit the work to the executor to avoid blocking the kazoo threads
+        # and queue(s)...
+        worker = self._worker
+        if worker is None:
+            return
         try:
-            self._worker.submit(self.notifier.notify, state, details)
-        except (AttributeError, RuntimeError):
-            # Notification thread is shutdown or non-existent, either case we
-            # just want to skip submitting a notification...
+            worker.submit(self.notifier.notify, state, details)
+        except RuntimeError:
+            # Notification thread is shutdown just skip submitting a
+            # notification...
             pass
 
     @property
@@ -481,7 +491,8 @@ class ZookeeperJobBoard(base.NotifyingJobBoard):
         LOG.debug("Got children %s under path %s", children, self.path)
         child_paths = []
         for c in children:
-            if c.endswith(LOCK_POSTFIX) or not c.startswith(JOB_PREFIX):
+            if (c.endswith(self.LOCK_POSTFIX) or
+                    not c.startswith(self.JOB_PREFIX)):
                 # Skip lock paths or non-job-paths (these are not valid jobs)
                 continue
             child_paths.append(k_paths.join(self.path, c))
@@ -665,7 +676,8 @@ class ZookeeperJobBoard(base.NotifyingJobBoard):
                 owner_data = self._get_owner_and_data(job)
                 lock_data, lock_stat, data, data_stat = owner_data
             except k_exceptions.NoNodeError:
-                raise excp.JobFailure("Can not consume a job %s"
+                excp.raise_with_cause(excp.JobFailure,
+                                      "Can not consume a job %s"
                                       " which we can not determine"
                                       " the owner of" % (job.uuid))
             if lock_data.get("owner") != who:
@@ -685,7 +697,8 @@ class ZookeeperJobBoard(base.NotifyingJobBoard):
                 owner_data = self._get_owner_and_data(job)
                 lock_data, lock_stat, data, data_stat = owner_data
             except k_exceptions.NoNodeError:
-                raise excp.JobFailure("Can not abandon a job %s"
+                excp.raise_with_cause(excp.JobFailure,
+                                      "Can not abandon a job %s"
                                       " which we can not determine"
                                       " the owner of" % (job.uuid))
             if lock_data.get("owner") != who:
@@ -703,17 +716,16 @@ class ZookeeperJobBoard(base.NotifyingJobBoard):
                 owner_data = self._get_owner_and_data(job)
                 lock_data, lock_stat, data, data_stat = owner_data
             except k_exceptions.NoNodeError:
-                raise excp.JobFailure("Can not trash a job %s"
+                excp.raise_with_cause(excp.JobFailure,
+                                      "Can not trash a job %s"
                                       " which we can not determine"
                                       " the owner of" % (job.uuid))
             if lock_data.get("owner") != who:
                 raise excp.JobFailure("Can not trash a job %s"
                                       " which is not owned by %s"
                                       % (job.uuid, who))
-
             trash_path = job.path.replace(self.path, self.trash_path)
             value = misc.binary_encode(jsonutils.dumps(data))
-
             txn = self._client.transaction()
             txn.create(trash_path, value=value)
             txn.delete(job.lock_path, version=lock_stat.version)
@@ -787,7 +799,7 @@ class ZookeeperJobBoard(base.NotifyingJobBoard):
                                   "Failed to connect to zookeeper")
         try:
             if self._conf.get('check_compatible', True):
-                kazoo_utils.check_compatible(self._client, MIN_ZK_VERSION)
+                kazoo_utils.check_compatible(self._client, self.MIN_ZK_VERSION)
             if self._worker is None and self._emit_notifications:
                 self._worker = futures.ThreadPoolExecutor(max_workers=1)
             self._client.ensure_path(self.path)
